@@ -57,7 +57,7 @@ static int test(struct args *args)
 	int const N = 8;
 	int const N_BUFFS = N * 64;
 	int const N_CQE_OVERFLOW = 4;
-	int const min_cqes = 2;
+	int const min_cqes = args->early_error ? 2 : 8;
 	int const NAME_LEN = sizeof(struct sockaddr_storage);
 	int const CONTROL_LEN = CMSG_ALIGN(sizeof(struct sockaddr_storage))
 					+ sizeof(struct cmsghdr);
@@ -165,7 +165,7 @@ static int test(struct args *args)
 		io_uring_prep_recv_multishot(sqe, fds[0], NULL, 0, 0);
 	}
 	sqe->flags |= IOSQE_BUFFER_SELECT;
-	sqe->buf_group = 7;
+        io_uring_sqe_set_buf_group(sqe, 7);
 	io_uring_sqe_set_data64(sqe, 1234);
 	io_uring_submit(&ring);
 
@@ -237,7 +237,11 @@ static int test(struct args *args)
 	usleep(1000);
 
 	if ((args->stream && !early_error) || recv_cqes < min_cqes) {
-		ret = io_uring_wait_cqes(&ring, &cqe, 1, &timeout, NULL);
+		unsigned int to_wait = 1;
+
+		if (recv_cqes < min_cqes)
+			to_wait = min_cqes - recv_cqes;
+		ret = io_uring_wait_cqes(&ring, &cqe, to_wait, &timeout, NULL);
 		if (ret && ret != -ETIME) {
 			fprintf(stderr, "wait final failed: %d\n", ret);
 			ret = -1;
@@ -499,7 +503,7 @@ static int test_enobuf(void)
 	assert(sqe);
 	io_uring_prep_recv_multishot(sqe, fds[0], NULL, 0, 0);
 	io_uring_sqe_set_data64(sqe, 1);
-	sqe->buf_group = 0;
+	io_uring_sqe_set_buf_group(sqe, 0);
 	sqe->flags |= IOSQE_BUFFER_SELECT;
 
 	ret = io_uring_submit(&ring);
